@@ -20,7 +20,7 @@
  */
 
 #include "ttf.h"
-#include <math.h>	// we need sqrt() or hypot() for normalisation
+#include <cmath>	// we need sqrt() or hypot() for normalisation
 
 #define FSHIFT 1.0
 #define MEMSLACK 2
@@ -37,7 +37,8 @@ Rasterizer::initInterpreter()
 
 	nPoints[0] = ttFont->maxpTable->maxTwilightPoints;
 	if (sizePoints[0] < nPoints[0]) {
-		if (sizePoints[0]) delete[] p[0];
+		if (sizePoints[0])
+			delete[] p[0];
 		sizePoints[0] = MEMSLACK * nPoints[0];
 		p[0] = new Point[sizePoints[0]];
 	}
@@ -47,25 +48,29 @@ Rasterizer::initInterpreter()
 		p[0][i] = nullpoint;
 
 	if (sizeStack < ttFont->maxpTable->maxStackSize) {
-		if (sizeStack) delete[] stackbase;
+		if (sizeStack)
+			delete[] stackbase;
 		sizeStack = MEMSLACK * ttFont->maxpTable->maxStackSize;
 		stackbase = stack = new int[sizeStack];
 	}
 
 	if (ttFont->cvtTable && sizeCvt < ttFont->cvtTable->numVals()) {
-		if (sizeCvt) delete[] cvt;
+		if (sizeCvt)
+			delete[] cvt;
 		sizeCvt	= MEMSLACK * ttFont->cvtTable->numVals();
 		cvt = new int[sizeCvt];
 	}
 
 	if (sizeStor < ttFont->maxpTable->maxStorage) {
-		if (sizeStor) delete[] stor;
+		if (sizeStor)
+			delete[] stor;
 		sizeStor = MEMSLACK * ttFont->maxpTable->maxStorage;
 		stor = new int[sizeStor];
 	}
 
 	if (sizeFDefs < ttFont->maxpTable->maxFunctionDefs) {
-		if (sizeFDefs) delete[] fdefs;
+		if (sizeFDefs)
+			delete[] fdefs;
 		sizeFDefs = MEMSLACK * ttFont->maxpTable->maxFunctionDefs;
 		fdefs = new FDefs[sizeFDefs];
 	}
@@ -93,16 +98,24 @@ Rasterizer::initInterpreter()
 void
 Rasterizer::endInterpreter()
 {
-	if (sizeIDefs)		delete[] idefs;
-	if (sizeFDefs)		delete[] fdefs;
-	if (sizeStor)		delete[] stor;
-	if (sizeCvt)		delete[] cvt;
-	if (sizeStack)		delete[] stackbase;
+	if (sizeIDefs)
+		delete[] idefs;
+	if (sizeFDefs)
+		delete[] fdefs;
+	if (sizeStor)
+		delete[] stor;
+	if (sizeCvt)
+		delete[] cvt;
+	if (sizeStack)
+		delete[] stackbase;
 /*
-	if (sizeContours)	delete[] endPoints;
-	if (sizePoints[1])	delete[] p[1];
+	if (sizeContours)
+		delete[] endPoints;
+	if (sizePoints[1])
+		delete[] p[1];
 */
-	if (sizePoints[0])	delete[] p[0];
+	if (sizePoints[0])
+		delete[] p[0];
 }
 
 void
@@ -205,7 +218,8 @@ Rasterizer::round(int x) const
 		debug("\tsround45(%d) = %d\t", x, (x<0)?-y:+y);
 		break;
 	}
-	if (y < 0) return 0;
+	if (y < 0)
+		return 0;
 	return (x >= 0) ? +y : -y;
 }
 
@@ -315,7 +329,8 @@ void
 GraphicsState::recalc()
 {
 	flags = f_vec_x ? X_TOUCHED : 0;
-	if (f_vec_y) flags |= Y_TOUCHED;
+	if (f_vec_y)
+		flags |= Y_TOUCHED;
 
 	int fp_cross = absNewMeasure(f_vec_x, f_vec_y);
 	if (fp_cross) {
@@ -623,8 +638,10 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 		n = (m >> 6) & 3;
 		gs.round_period = 0x20 << n;
 		n = (m >> 4) & 3;
-		if (n == 3) gs.round_phase = 48;
-		else gs.round_phase = (gs.round_period >> 2) * n;
+		if (n == 3)
+			gs.round_phase = 48;
+		else
+			gs.round_phase = (gs.round_period >> 2) * n;
 		m &= 0x0F;
 		if (m)
 			gs.round_thold = (gs.round_period >> 3) * (m - 4);
@@ -666,12 +683,18 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 	case SCANCTRL:
 		m = *(stack--);
 		gs.dropout_control = 0;
-		if (m & 0x0100 && mppem <= (m & 0xff))	gs.dropout_control = 1;
-		if (m & 0x0200 && !xy && !yx)		gs.dropout_control = 1;
-		if (m & 0x0400 && xx != yy)		gs.dropout_control = 1;
-		if (m & 0x0800 && mppem > (m & 0xff))	gs.dropout_control = 0;
-		if (m & 0x1000 && (xy || yx))		gs.dropout_control = 0;
-		if (m & 0x2000 && xx == yy)		gs.dropout_control = 0;
+		if (m & 0x0100 && mppem <= (m & 0xff))
+			gs.dropout_control = 1;
+		if (m & 0x0200 && !xy && !yx)
+			gs.dropout_control = 1;
+		if (m & 0x0400 && xx != yy)
+			gs.dropout_control = 1;
+		if (m & 0x0800 && mppem > (m & 0xff))
+			gs.dropout_control = 0;
+		if (m & 0x1000 && (xy || yx))
+			gs.dropout_control = 0;
+		if (m & 0x2000 && xx == yy)
+			gs.dropout_control = 0;
 		debug("SCANCTRL %04X -> %d", m, gs.dropout_control);
 		break;
 	case SCANTYPE:
@@ -754,7 +777,7 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 		n = *stack;
 		debug("MD1 p[%d][%d] ", gs.zp1 == p[1], m);
 		debug("- p[%d][%d]", gs.zp0 == p[1], n);
-		*stack = oldMeasure(gs.zp0[n], gs.zp1[m]); // Thanks David
+		*stack = oldMeasure(gs.zp0[n], gs.zp1[m]);
 		break;
 	case MPPEM:
 		debug("MPPEM\t");
@@ -834,7 +857,8 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 		n = gs.absNewMeasure(pp->xnow - pp->xold, pp->ynow - pp->yold);
 		assert(m >= 0 && m <= 1);
 		for (Point *pp1 = p[m], *pp2 = pp1 + nPoints[m]; pp1 < pp2; ++pp1) {
-			if (pp1 == pp) continue;
+			if (pp1 == pp)
+				continue;
 			debug("\nSHZ p[%zd] by %f", pp1 - p[m], n / FSHIFT);
 			debug("\t(%d %d) -> ", pp1->xnow, pp1->ynow);
 			pp1->xnow += (n * gs.move_x) >> 14;
@@ -1058,9 +1082,9 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 		debug("and p[%zd]-p[%zd] ", pp3 - gs.zp0, pp4 - gs.zp0);
 
 		int f1 = (pp1->xnow - pp3->xnow) * (pp4->ynow - pp3->ynow) -
-			 (pp1->ynow - pp3->ynow) * (pp4->xnow - pp3->xnow);
+		         (pp1->ynow - pp3->ynow) * (pp4->xnow - pp3->xnow);
 		int f2 = (pp2->ynow - pp1->ynow) * (pp4->xnow - pp3->xnow) -
-			 (pp2->xnow - pp1->xnow) * (pp4->ynow - pp3->ynow);
+		         (pp2->xnow - pp1->xnow) * (pp4->ynow - pp3->ynow);
 
 		pp3 = &gs.zp2[m];
 		pp3->flags |= X_TOUCHED | Y_TOUCHED;
@@ -1070,9 +1094,9 @@ Rasterizer::execOpcode(RandomAccessFile* const f)
 			debug("are parallel!\n");
 		} else {
 			pp3->xnow = pp1->xnow +
-				    MULDIV(f1, pp2->xnow - pp1->xnow, f2);
+			            MULDIV(f1, pp2->xnow - pp1->xnow, f2);
 			pp3->ynow = pp1->ynow +
-				    MULDIV(f1, pp2->ynow - pp1->ynow, f2);
+			            MULDIV(f1, pp2->ynow - pp1->ynow, f2);
 		}
 
 		debug("\n-> %d %d", pp3->xnow, pp3->ynow);
@@ -1138,6 +1162,7 @@ deltap_label:
 		while (--m >= 0) {
 			int pno = *(stack--);
 			int arg = *(stack--);
+
 			debug("\np[%d] arg %04X", pno, arg);
 			debug("\targ.n=%d, n=%d", arg >> 4, n >> 4);
 			if (n > (arg & 0xf0))
@@ -1203,7 +1228,8 @@ deltac_label:
 				break;
 			if (n == (arg & 0xf0)) {
 				arg = (arg & 15) - 8;
-				if (arg >= 0) ++arg;
+				if (arg >= 0)
+					++arg;
 				arg <<= SHIFT - gs.delta_shift;
 				debug("\tmoved by %f,\t%d ",
 				      arg / FSHIFT, cvt[cno]);
@@ -1403,7 +1429,8 @@ jump_relative:
 		break;
 	case ABS:
 		m = *stack;
-		if (m < 0) *stack = -m;
+		if (m < 0)
+			*stack = -m;
 		debug("ABS %d = %d", m, *stack);
 		break;
 	case NEG:
@@ -1521,7 +1548,7 @@ jump_relative:
 			debug("IDEF_CALL 0x%02X, ofs = %05X, len = %d\n",
 			      opc, idef->offset, idef->length);
 
-			if (idef->length) // Thanks Colin McCormack
+			if (idef->length)
 				execHints(idef->f, idef->offset, idef->length);
 			else
 				debug("illegal instruction %02X\n", opc);
@@ -1618,7 +1645,7 @@ Rasterizer::interpolate(Point &pp, const Point &p2, const Point &p1)
 	if ((dold21 ^ doldp1) < 0 || doldp1 == 0)
 		dist = newMeasure(p1, pp) + doldp1; //- oldMeasure(p1, pp);
 	else if ((dold21 >= 0 && doldp1 >= dold21) ||
-		 (dold21 <= 0 && doldp1 <= dold21))
+	         (dold21 <= 0 && doldp1 <= dold21))
 		dist = newMeasure(p2, pp) - oldMeasure(p2, pp);
 	else {
 		int dnew21 = newMeasure(p2, p1);
