@@ -2,7 +2,7 @@
  * Show glyphs
  *
  * Copyright © 1997-1998 Herbert Duerr
- * Copyright © 2004,2008,2010,2012 Guillem Jover <guillem@hadrons.org>
+ * Copyright © 2004, 2008-2012, 2018-2019 Guillem Jover <guillem@hadrons.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,10 +25,10 @@
 #define STARTGLYF	6
 #define DEFAULT_FONT	"times.ttf"
 
-#include <math.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cmath>
+#include <cstring>
+#include <cstdio>
+#include <cstdlib>
 
 #include <X11/keysym.h>
 #include <xcb/xcb.h>
@@ -227,7 +227,7 @@ main(int argc, char** argv)
 
 	gc = xcb_generate_id(c);
 
-	uint32_t mask;
+	uint32_t mask = 0;
 	uint32_t values[5];
 
 	mask |= XCB_GC_FUNCTION;
@@ -297,8 +297,8 @@ main(int argc, char** argv)
 	xcb_flush(c);
 
 	raster = new Rasterizer();
-	TTFont *ttFont = new TTFont(ttFileName);
-	raster->useTTFont(ttFont);
+	TTFont ttFont(ttFileName);
+	raster->useTTFont(&ttFont);
 
 	state.size = 16;
 	state.angle = 0;
@@ -317,7 +317,7 @@ main(int argc, char** argv)
 			}
 #endif
 		{
-			//ttFont->getGlyphNo16(state.glyphNo);
+			//ttFont.getGlyphNo16(state.glyphNo);
 			fprintf(stderr, "gno2 %d\n", state.glyphNo);
 			fflush(stderr);
 			glyph2image(&state, &img);
@@ -422,10 +422,10 @@ main(int argc, char** argv)
 			case XK_Control_R:
 				goto expose;
 			default:
-				// XXX: glyphNo = ttFont->getGlyphNo16(0x20AC);
+				// XXX: glyphNo = ttFont.getGlyphNo16(0x20AC);
 
 				// FIXME: need keysym name.
-				state.glyphNo = ttFont->getGlyphNo16(ksym);
+				state.glyphNo = ttFont.getGlyphNo16(ksym);
 				// printf("key = \"%s\" -> %d\n", keyname, glyphNo);
 				printf("key = %c -> %d\n", ksym, state.glyphNo);
 				glyph2image(&state, &img);
@@ -458,7 +458,6 @@ main(int argc, char** argv)
 		deallocMem(bitmap, BMPSIZE);
 
 	delete raster;
-	delete ttFont;
 	delete [] pixmap;
 
 	cleanupMem();
