@@ -3,7 +3,7 @@
  *
  * Copyright © 1997-1999 Herbert Duerr
  * Copyright © 1999 Stephen Carpenter and others
- * Copyright © 2002-2008 Guillem Jover
+ * Copyright © 2002-2012, 2016-2020 Guillem Jover
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,7 +39,6 @@
 #define MINFONTBUFSIZE	(1 << 18)
 
 #include "gettext.h"
-#include <locale.h>
 #define _(str) gettext(str)
 #define N_(str) gettext_noop(str)
 #include "ttf.h"
@@ -49,27 +48,32 @@
 #include "mesg.h"
 
 #include <sys/types.h>
-#include <dirent.h>
-#include <string.h>
-#include <string>
-#include <ctype.h>
-
-#include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <signal.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+
 #include <netinet/in.h>
+
+#include <dirent.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <time.h>
+#include <pwd.h>
+#include <unistd.h>
+
+#include <string>
+#include <vector>
+#include <cerrno>
+#include <clocale>
+#include <cctype>
+#include <cstring>
+#include <cstdlib>
+#include <csignal>
+
 #include <X11/fonts/FS.h>
 #include <X11/fonts/FSproto.h>
-#include <pwd.h>
-#include <errno.h>
 
 // if you want to read good code skip this hacked up file!
 
@@ -104,9 +108,7 @@ struct fs_conn {
 	bool listen_inet;
 	int port;
 	int sd_max;
-	int *sd_list;
-	int sd_list_size;
-	int sd_list_used;
+	std::vector<int> sd_list;
 };
 
 #define MAXREQSIZE 4096
@@ -131,8 +133,7 @@ static gid_t newgid = (gid_t)(-2);
 static const char *sockname;
 static const char *sockdir = "/tmp/.font-unix";
 
-#define MAXENC 16	/* Maximum number of encodings */
-static Encoding *encodings[MAXENC];
+static EncodingsActive encodings;
 
 static void
 version()
@@ -184,7 +185,7 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 	int ttdir_len = strlen(ttdir);
 
 	if (!gslist)
-		info(_("Sync in directory \"%s/%s\".\n"), fontdir, ttdir);
+		info(_("Sync in directory \"%s/%s\"."), fontdir, ttdir);
 
 	DIR *dirp = opendir(".");
 	if (dirp == nullptr)
@@ -214,14 +215,12 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 		char pathName[ttdir_len + namelen + 2];
 		sprintf(pathName, "%s/%s", ttdir, de->d_name);
 
-		TTFont *ttFont = new TTFont(de->d_name, 1);
-		if (!ttFont || ttFont->badFont()) {
-			delete ttFont;
+		TTFont ttFont(de->d_name, 1);
+		if (ttFont.badFont())
 			continue;
-		}
 
 		FontInfo fi;
-		ttFont->getFontInfo(&fi);
+		ttFont.getFontInfo(&fi);
 
 		TTFNdata info = { 0 };
 		info.nameOfs = ftell(nameFile);
@@ -243,7 +242,7 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 		else
 			xlfd_templ += ttdir;
 
-		string xlfd = ttFont->getXLFDbase(xlfd_templ);
+		string xlfd = ttFont.getXLFDbase(xlfd_templ);
 		info.xlfdLen = xlfd.length();
 		fwrite((void *)xlfd.c_str(), 1, info.xlfdLen, nameFile);
 		fputc('\0', nameFile);
@@ -263,7 +262,6 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, const char *ttdir, bool gslist)
 
 		fwrite((void *)&info, 1, sizeof(info), infoFile);
 
-		delete ttFont;
 		++nfonts;
 	}
 	closedir(dirp);
@@ -277,11 +275,11 @@ cachefile(string leafname)
 	struct stat statbuf;
 
 	if (stat(cachedir, &statbuf)) {
-		error(_("directory \"%s\" does not exist!\n"), cachedir);
+		error(_("directory \"%s\" does not exist!"), cachedir);
 		return string();
 	}
 	if (!S_ISDIR(statbuf.st_mode)) {
-		error(_("\"%s\" is not a directory!\n"), cachedir);
+		error(_("\"%s\" is not a directory!"), cachedir);
 		return string();
 	}
 
@@ -295,7 +293,7 @@ ttSyncAll(bool gslist = false)
 		debug("TrueType syncing\n");
 
 	if (chdir(fontdir)) {
-		error(_("directory \"%s\" does not exist!\n"), fontdir);
+		error(_("directory \"%s\" does not exist!"), fontdir);
 		return -1;
 	}
 
@@ -307,54 +305,54 @@ ttSyncAll(bool gslist = false)
 		return -1;
 	}
 
-	FILE *infoFile = fopen(ttinfofilename.c_str(), "wb");
-	FILE *nameFile = fopen(ttnamefilename.c_str(), "wb");
+	FILE *ttinfoFile = fopen(ttinfofilename.c_str(), "wb");
+	FILE *ttnameFile = fopen(ttnamefilename.c_str(), "wb");
 
-	if (infoFile == nullptr || nameFile == nullptr) {
-		if (infoFile)
-			fclose(infoFile);
-		if (nameFile)
-			fclose(nameFile);
-		error(_("cannot write to font database!\n"));
+	if (ttinfoFile == nullptr || ttnameFile == nullptr) {
+		if (ttinfoFile)
+			fclose(ttinfoFile);
+		if (ttnameFile)
+			fclose(ttnameFile);
+		error(_("cannot write to font database!"));
 		return -1;
 	}
 
-	TTFNheader info;
-	memcpy(info.magic, "TTFN", 4);
-	info.version = TTFN_VERSION;
-	info.key = 0;	// XXX
-	info.crc = 0;	// XXX
-	memcpy(info.type, "INFO", 4);
-	fwrite((void *)&info, 1, sizeof(info), infoFile);
-	memcpy(info.type, "NAME", 4);
-	fwrite((void *)&info, 1, sizeof(info), nameFile);
+	TTFNheader ttinfo;
+	memcpy(ttinfo.magic, "TTFN", 4);
+	ttinfo.version = TTFN_VERSION;
+	ttinfo.key = 0;	// XXX
+	ttinfo.crc = 0;	// XXX
+	memcpy(ttinfo.type, "INFO", 4);
+	fwrite((void *)&ttinfo, 1, sizeof(ttinfo), ttinfoFile);
+	memcpy(ttinfo.type, "NAME", 4);
+	fwrite((void *)&ttinfo, 1, sizeof(ttinfo), ttnameFile);
 
-	int nfonts = ttSyncDir(infoFile, nameFile, ".", gslist);
+	int nfonts = ttSyncDir(ttinfoFile, ttnameFile, ".", gslist);
 
 	DIR *dirp = opendir(".");
 	if (dirp == nullptr) {
-		fclose(infoFile);
-		fclose(nameFile);
+		fclose(ttinfoFile);
+		fclose(ttnameFile);
 		return 0;
 	}
 
 	while (dirent *de = readdir(dirp)) {
 		chdir(fontdir);
 		if (de->d_name[0] != '.' && !chdir(de->d_name))
-			nfonts += ttSyncDir(infoFile, nameFile,
-					    de->d_name, gslist);
+			nfonts += ttSyncDir(ttinfoFile, ttnameFile,
+			                    de->d_name, gslist);
 	}
 	closedir(dirp);
 
-	fclose(infoFile);
-	fclose(nameFile);
+	fclose(ttinfoFile);
+	fclose(ttnameFile);
 
 	if (nfonts > 0) {
 		if (!gslist)
-			info(_("Found %d fonts.\n"), nfonts);
+			info(_("Found %d fonts."), nfonts);
 	} else {
-		error(_("no valid truetype fonts found!\n"));
-		info(_("Please put some *.ttf fonts into \"%s\".\n"), fontdir);
+		error(_("no valid truetype fonts found!"));
+		info(_("Please put some *.ttf fonts into \"%s\"."), fontdir);
 	}
 
 	return nfonts;
@@ -370,8 +368,9 @@ listXLFDFonts(char *pattern0, int index, char *buf)
 	if (index == 0) {
 		ttfn = (TTFNdata *)(infoBase + sizeof(TTFNheader));
 		mapIndex = 0;
-	} else if (mapIndex == 0 && (char *)++ttfn >= infoBase + infoSize)
+	} else if (mapIndex == 0 && (char *)++ttfn >= infoBase + infoSize) {
 		return -1;
+	}
 
 	char *pattern = pattern0;
 
@@ -381,8 +380,8 @@ listXLFDFonts(char *pattern0, int index, char *buf)
 	char proportion = (ttfn->bProportion == 9) ? 'm' : 'p';
 
 	char *buf0 = buf++;
-	if (pattern[0] == '*' // Thanks CK
-	    || (pattern[0] == '-' && pattern[1] == '*' && pattern[2] == 0)) {
+	if (pattern[0] == '*' ||
+	    (pattern[0] == '-' && pattern[1] == '*' && pattern[2] == 0)) {
 		char xlfdExt[] = "0-0-0-0-p-0-iso8859-1";
 		xlfdExt[8] = proportion;
 		strcpy(buf, xlfdName);
@@ -418,14 +417,15 @@ listXLFDFonts(char *pattern0, int index, char *buf)
 	}
 
 	*buf = 0;
-	// XXX: this hack satisfies Mozilla, thanks Andrew Turner
+	// XXX: This hack satisfies Mozilla.
 	if (!strcmp(buf - 4, "-0-0")) {
 		strcpy(buf - 3, encodings[mapIndex]->Name.c_str());
 		buf += encodings[mapIndex]->Name.size() - 3;
 		if (!encodings[++mapIndex])
 			mapIndex = 0;
-	} else
+	} else {
 		mapIndex = 0;
+	}
 
 	debug("match\t\"%s\"\n", buf0 + 1);
 
@@ -443,8 +443,9 @@ listTTFNFonts(char *pattern, int index, char *buf)
 
 	if (index == 0 || ttfn == nullptr) {
 		ttfn = (TTFNdata *)(infoBase + sizeof(TTFNheader));
-	} else if ((char *)++ttfn >= infoBase + infoSize)
+	} else if ((char *)++ttfn >= infoBase + infoSize) {
 		return -1;
+	}
 
 	char *fontName = nameBase + ttfn->nameOfs;
 
@@ -453,10 +454,10 @@ listTTFNFonts(char *pattern, int index, char *buf)
 	/* Pre-allocate the buffer so that the terminating '\0' fits. */
 	char panose[sizeof(fn.panose) + 1];
 	sprintf(panose, "%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X",
-		ttfn->bFamilyType, ttfn->bSerifStyle, ttfn->bWeight,
-		ttfn->bProportion, ttfn->bContrast, ttfn->bStrokeVariation,
-		ttfn->bArmStyle, ttfn->bLetterForm, ttfn->bMidLine,
-		ttfn->bXHeight);
+	        ttfn->bFamilyType, ttfn->bSerifStyle, ttfn->bWeight,
+	        ttfn->bProportion, ttfn->bContrast, ttfn->bStrokeVariation,
+	        ttfn->bArmStyle, ttfn->bLetterForm, ttfn->bMidLine,
+	        ttfn->bXHeight);
 	/* Copy only the text, not the terminating '\0'. */
 	memcpy(&fn.panose[0][0], panose, sizeof(fn.panose));
 
@@ -503,7 +504,7 @@ fs_find_font(Font fid, fs_client &client)
 
 static XFSFont *
 openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
-	 int fid, Encoding *encoding)
+         int fid, Encoding *encoding)
 {
 	debug("point %d, pixel %d, res %d\n",
 	      fp->point[0], fp->pixel[0], fp->resolution[0]);
@@ -574,7 +575,7 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 	xfs->fe.buflen = MAXFONTBUFSIZE;
 	while (!(xfs->fe.buffer = (uint8_t *)allocMem(xfs->fe.buflen)))
 		if ((xfs->fe.buflen >>= 1) < MINFONTBUFSIZE) {
-			error(_("entering memory starved mode.\n"));
+			error(_("entering memory starved mode"));
 			xfs->fid = 0;	// Mark font slot as not used.
 			delete ttFont;
 			return nullptr;
@@ -694,7 +695,7 @@ openTTFN(Rasterizer *raster, char *ttfnName, FontParams *fp, int fid)
 		if (!strcmp(name, ttfnName)) {
 			chdir(fontdir);
 			return openFont(new TTFont(file), fp, raster, fid,
-					encodings[0]);
+			                encodings[0]);
 		}
 	}
 	return nullptr;
@@ -737,7 +738,7 @@ openXLFD(Rasterizer *raster, char *xlfdName, FontParams *fp, int fid)
 			case 13:
 				for (char *cp = p; *cp; ++cp)
 					*cp = tolower(*cp);
-				encoding = Encoding::find(++p);
+				encoding = EncodingsRegistry::find(++p);
 				break;
 			}
 	}
@@ -767,7 +768,7 @@ openXLFD(Rasterizer *raster, char *xlfdName, FontParams *fp, int fid)
 		if (*p == 0 && *xlfd == 0) {
 			chdir(fontdir);
 			return openFont(new TTFont(file), fp, raster, fid,
-					encoding);
+			                encoding);
 		}
 	}
 
@@ -781,7 +782,7 @@ openTTFdb()
 	infoSize = nameSize = 0;
 
 	if (chdir(fontdir)) {
-		error(_("directory \"%s\" does not exist!\n"), fontdir);
+		error(_("directory \"%s\" does not exist!"), fontdir);
 		return 0;
 	}
 
@@ -791,13 +792,13 @@ openTTFdb()
 
 	int fd = open(ttinfofilename.c_str(), O_RDONLY);
 	if (fd < 0) {
-		error(_("cannot open font database!\n"));
+		error(_("cannot open font database!"));
 		return 0;
 	}
 
 	struct stat statbuf;
 	if (fstat(fd, &statbuf) < 0) {
-		error(_("cannot stat fond database!\n"));
+		error(_("cannot stat fond database!"));
 		close(fd);
 		return 0;
 	}
@@ -806,18 +807,18 @@ openTTFdb()
 	close(fd);
 
 	if (infoBase == MAP_FAILED) {
-		error(_("cannot mmap font database!\n"));
+		error(_("cannot mmap font database!"));
 		return 0;
 	}
 
-	if (infoSize <= sizeof(TTFNheader)
-	    || strncmp(infoBase, "TTFNINFO", 8)) {
-		error(_("corrupt font database!\n"));
+	if (infoSize <= sizeof(TTFNheader) ||
+	    strncmp(infoBase, "TTFNINFO", 8)) {
+		error(_("corrupt font database!"));
 		return 0;
 	}
 
 	if (((TTFNheader *)infoBase)->version != TTFN_VERSION) {
-		error(_("wrong font database version!\n"));
+		error(_("wrong font database version!"));
 		return 0;
 	}
 
@@ -827,12 +828,12 @@ openTTFdb()
 
 	fd = open(ttnamefilename.c_str(), O_RDONLY);
 	if (fd < 0) {
-		error(_("cannot open font database!\n"));
+		error(_("cannot open font database!"));
 		return 0;
 	}
 
 	if (fstat(fd, &statbuf) < 0) {
-		error(_("cannot stat font database!\n"));
+		error(_("cannot stat font database!"));
 		close(fd);
 		return 0;
 	}
@@ -841,18 +842,18 @@ openTTFdb()
 	close(fd);
 
 	if (nameBase == MAP_FAILED) {
-		error(_("cannot mmap font database!\n"));
+		error(_("cannot mmap font database!"));
 		return 0;
 	}
 
-	if (nameSize <= sizeof(TTFNheader)
-	    || strncmp(nameBase, "TTFNNAME", 8)) {
-		error(_("corrupt font database!\n"));
+	if (nameSize <= sizeof(TTFNheader) ||
+	    strncmp(nameBase, "TTFNNAME", 8)) {
+		error(_("corrupt font database!"));
 		return 0;
 	}
 
 	if (((TTFNheader *)nameBase)->version != TTFN_VERSION) {
-		error(_("wrong font database version!\n"));
+		error(_("wrong font database version!"));
 		return 0;
 	}
 
@@ -886,7 +887,7 @@ fs_connection_setup_inet(fs_conn &conn, struct addrinfo *res)
 	int inet_ports = 0;
 	const int on = 1;
 
-	for (; res && conn.sd_list_used < conn.sd_list_size; res = res->ai_next) {
+	for (; res; res = res->ai_next) {
 		int sd;
 
 		sd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
@@ -896,14 +897,14 @@ fs_connection_setup_inet(fs_conn &conn, struct addrinfo *res)
 #if defined(HAVE_IPV6_V6ONLY)
 		if (res->ai_family == PF_INET6 &&
 		    setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0) {
-			error(_("setting socket option (IPv6 only).\n"));
+			error(_("setting socket option (IPv6 only)"));
 			close(sd);
 			continue;
 		}
 #endif
 
 		if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
-			error(_("setting socket option (reuseaddr).\n"));
+			error(_("setting socket option (reuseaddr)"));
 			close(sd);
 			continue;
 		}
@@ -914,12 +915,12 @@ fs_connection_setup_inet(fs_conn &conn, struct addrinfo *res)
 		}
 
 		listen(sd, 1);
-		conn.sd_list[conn.sd_list_used++] = sd;
+		conn.sd_list.push_back(sd);
 		inet_ports++;
 	}
 
 	if (!inet_ports) {
-		error(_("cannot open TCP/IP port %d, try another port; %s!\n"),
+		error(_("cannot open TCP/IP port %d, try another port; %s!"),
 		      conn.port, strerror(errno));
 		return false;
 	} else
@@ -927,6 +928,56 @@ fs_connection_setup_inet(fs_conn &conn, struct addrinfo *res)
 }
 #endif
 
+static int
+fs_connection_mkdir()
+{
+	struct stat st;
+
+	if (lstat(sockdir, &st) < 0) {
+		if (errno != ENOENT) {
+			error(_("cannot get metadata for socket directory %s"),
+			      sockdir);
+			return -1;
+		}
+
+		if (geteuid() != 0) {
+			error(_("cannot create socket directory %s owned by root"),
+			      sockdir);
+			return -1;
+		}
+
+		if (mkdir(sockdir, 01777) < 0) {
+			error(_("cannot create socket directory %s"), sockdir);
+			return -1;
+		}
+	} else {
+		if (!S_ISDIR(st.st_mode)) {
+			error(_("pre-existing socket directory %s is not a directory"),
+			      sockdir);
+			return -1;
+		}
+
+		if (st.st_uid != 0) {
+			error(_("pre-existing socket directory %s not owned by root"),
+			      sockdir);
+			return -1;
+		}
+
+		if (~st.st_mode & 0022) {
+			warning(_("pre-existing socket directory %s is not world-writable %#o"),
+			        sockdir, st.st_mode & ~S_IFMT);
+		}
+
+		if ((st.st_mode & 01000) == 0) {
+			error(_("pre-existing socket directory %s is missing the sticky bit %#o"),
+			      sockdir, st.st_mode & ~S_IFMT);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
 static int
 fs_connection_setup(fs_conn &conn)
 {
@@ -936,34 +987,31 @@ fs_connection_setup(fs_conn &conn)
 		struct sockaddr_un s_unix;
 		mode_t old_umask;
 
+		s_unix.sun_family = AF_UNIX;
+		sprintf(s_unix.sun_path, "%s/fs%d", sockdir, conn.port);
+		sockname = strdup(s_unix.sun_path);
+
+		old_umask = umask(0);
+		if (fs_connection_mkdir() < 0)
+			return -1;
+		unlink(s_unix.sun_path);
+
 		// prepare unix connection
 		sd = socket(PF_UNIX, SOCK_STREAM, 0);
 		if (sd < 0) {
-			error(_("cannot create Unix socket; %s!\n"),
+			error(_("cannot create Unix socket; %s!"),
 			      strerror(errno));
 			return -1;
 		}
 
-		s_unix.sun_family = AF_UNIX;
-		sprintf(s_unix.sun_path, "%s/fs%d", sockdir, conn.port);
-		sockname = s_unix.sun_path;
-
-		old_umask = umask(0);
-		if (mkdir(sockdir, 01777) < 0) {
-			error(_("cannot make socket directory %s!\n"), sockdir);
-			close(sd);
-			return -1;
-		}
-		unlink(s_unix.sun_path);
-
 		if (bind(sd, (struct sockaddr *)&s_unix, sizeof(s_unix)) < 0) {
-			error(_("could not write to %s/, please check "
-			      "permissions.\n"), sockdir);
+			error(_("could not write to %s, please check "
+			        "permissions"), sockname);
 			close(sd);
 			return -1;
 		} else {
 			listen(sd, 1);
-			conn.sd_list[conn.sd_list_used++] = sd;
+			conn.sd_list.push_back(sd);
 		}
 		umask(old_umask);
 	}
@@ -1004,24 +1052,24 @@ fs_connection_setup(fs_conn &conn)
 
 		if (bind(sd, (struct sockaddr *)&s_inet, sizeof(s_inet)) < 0) {
 			error(_("cannot open TCP/IP port %d, "
-				"try another port; %s!\n"), conn.port,
+				"try another port; %s!"), conn.port,
 				strerror(errno));
 			close(sd);
 			return -1;
 		}
 		listen(sd, 1);
-		conn.sd_list[conn.sd_list_used++] = sd;
+		conn.sd_list.push_back(sd);
 #endif
 	}
 
 	conn.sd_max = 0;
 
-	for (int n = 0; n < conn.sd_list_used; n++) {
+	for (int n = 0; n < conn.sd_list.size(); n++) {
 		if (conn.sd_list[n] > conn.sd_max)
 			conn.sd_max = conn.sd_list[n];
 	}
 
-	debug("connection setup (sockets = %d)\n", conn.sd_list_used);
+	debug("connection setup (sockets = %d)\n", conn.sd_list.size());
 
 	return 0;
 }
@@ -1034,14 +1082,14 @@ fs_connection_new(fs_conn &conn)
 
 	FD_ZERO(&sd_set);
 
-	for (n = 0; n < conn.sd_list_used; n++)
+	for (n = 0; n < conn.sd_list.size(); n++)
 		FD_SET(conn.sd_list[n], &sd_set);
 
 	select(conn.sd_max + 1, &sd_set, nullptr, nullptr, nullptr);
 
 	int sd = 0;
 
-	for (n = 0; n < conn.sd_list_used; n++) {
+	for (n = 0; n < conn.sd_list.size(); n++) {
 		if (FD_ISSET(conn.sd_list[n], &sd_set)) {
 			sd = accept(conn.sd_list[n], nullptr, nullptr);
 			break;
@@ -1067,7 +1115,7 @@ fs_connecting(fs_client &client)
 		return 0;
 
 	if (req->byteOrder != 'l' && req->byteOrder != 'B') {
-		error(_("invalid byteorder, giving up.\n"));
+		error(_("invalid byteorder, giving up"));
 		return 0;
 	}
 
@@ -1078,7 +1126,7 @@ fs_connecting(fs_client &client)
 	if ((req->byteOrder == 'l' && (*(uint32_t *)req & 0xff) != 'l') ||
 	    (req->byteOrder == 'B' && ((*(uint32_t *)req >> 24) & 0xff) != 'B'))
 	{
-		error(_("byteorder mismatch, giving up.\n"));
+		error(_("byteorder mismatch, giving up"));
 		return 0;
 	}
 
@@ -1116,7 +1164,7 @@ fixup_bitmap(FontExtent *fe, uint32_t hint)
 	int format = ((hint >> 8) & 3) + 3;
 	if (format < LOGSLP) {
 		error(_("scanline length error! recompile xfstt with LOGSLP "
-		      "defined as %d!\n"), format <= 3 ? 3 : format);
+		        "defined as %d!"), format <= 3 ? 3 : format);
 	}
 
 	if ((hint ^ fe->bmpFormat) == 0)
@@ -1149,8 +1197,10 @@ fixup_bitmap(FontExtent *fe, uint32_t hint)
 
 	if ((fe->bmpFormat ^ hint) & BitmapFormatMaskBit) {
 		debug("bitswap\n");
-		uint8_t map[16] = {0, 8, 4, 12, 2, 10, 6, 14,
-			      1, 9, 5, 13, 3, 11, 7, 15};
+		uint8_t map[16] = {
+			0, 8, 4, 12, 2, 10, 6, 14,
+			1, 9, 5, 13, 3, 11, 7, 15,
+		};
 		for (p = fe->bitmaps; p < end; ++p)
 			*p = (map[*p & 15] << 4) | map[(*p >> 4) & 15];
 	}
@@ -1723,7 +1773,6 @@ fs_query_x_extents(fs_client &client)
 		ext->descent = gm->yOrigin;
 		ext->attributes = gm->yAdvance;
 
-		// Thanks GB
 		if (!glyphNo && ch != xfs->fi.firstChar) {
 			ext->left = ext->right = 0;
 			ext->ascent = ext->descent = 0;
@@ -1968,18 +2017,24 @@ fs_working(fs_client &client, Rasterizer *raster)
 	return 0;
 }
 
-/* This is a cheesy little signal handler to make sure that the
- * pid file is properly disposed of when we are killed
- * possibly a better (more robust) signal handler could be written - sjc
- */
 static void
-delPIDfile(int signal XFSTT_ATTR_UNUSED)
+server_cleanup()
 {
-	unlink(pidfilename);
+	debug("xfstt: cleaning up\n");
+	if (daemon)
+		unlink(pidfilename);
 	if (sockname) {
-		unlink(sockname);
+		if (unlink(sockname) < 0)
+			error(_("cannot remove sockfile %s: %s"), sockname,
+			      strerror(errno));
 		rmdir(sockdir);
 	}
+}
+
+static void
+sigterm_handler(int signal XFSTT_ATTR_UNUSED)
+{
+	server_cleanup();
 	exit(0);
 }
 
@@ -2029,14 +2084,9 @@ main(int argc, char **argv)
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
-	Encoding::getDefault(encodings, MAXENC);
-
 	fs_conn.listen_unix = true;
 	fs_conn.listen_inet = true;
 	fs_conn.port = default_port;
-	fs_conn.sd_list_size = 16;
-	fs_conn.sd_list_used = 0;
-	fs_conn.sd_list = new int [fs_conn.sd_list_size];
 
 	for (int i = 1; i < argc; ++i) {
 		if (!strcmp(argv[i], "--gslist")) {
@@ -2047,7 +2097,7 @@ main(int argc, char **argv)
 			if (i <= argc)
 				fs_conn.port = xatoi(argv[++i]);
 			if (!fs_conn.port) {
-				error(_("illegal port number!\n"));
+				error(_("illegal port number!"));
 				fs_conn.port = default_port;
 			}
 		} else if (!strcmp(argv[i], "--notcp")) {
@@ -2056,7 +2106,7 @@ main(int argc, char **argv)
 			if (i <= argc)
 				defaultres = xatoi(argv[++i]);
 			if (!defaultres)
-				error(_("illegal default resolution!\n"));
+				error(_("illegal default resolution!"));
 		} else if (!strcmp(argv[i], "--dir")) {
 			fontdir = argv[++i];
 		} else if (!strcmp(argv[i], "--user")) {
@@ -2067,23 +2117,23 @@ main(int argc, char **argv)
 			cachedir = argv[++i];
 		} else if (!strcmp(argv[i], "--encoding")) {
 			char *maplist = argv[++i];
-			if (!Encoding::parse(maplist, encodings, MAXENC)) {
-				error(_("illegal encoding!\n"));
-				info(_("valid encodings are:\n"));
-				for (Encoding *maps = nullptr;;) {
-					maps = Encoding::enumerate(maps);
-					if (!maps)
-						exit(0);
-					info("\t%s\n", maps->Name.c_str());
-				}
+			if (encodings.parse(maplist) > 0)
+				continue;
+
+			error(_("illegal encoding!"));
+			info(_("valid encodings are:"));
+			EncodingsRegistry::iterator iter;
+			for (iter = EncodingsRegistry::begin(); iter != EncodingsRegistry::end(); ++iter) {
+				info("\t%s", (*iter)->Name.c_str());
 			}
+			exit(0);
 		} else if (!strcmp(argv[i], "--help")) {
 			usage(1);
 			return 0;
 		} else if (!strcmp(argv[i], "--version")) {
 			version();
 			return 0;
-		} else if (!strcmp(argv[i], "--inetd")) { // thanks Feanor
+		} else if (!strcmp(argv[i], "--inetd")) {
 			inetdConnection = true;
 			multiConnection = false;
 		} else if (!strcmp(argv[i], "--once")) {
@@ -2091,7 +2141,7 @@ main(int argc, char **argv)
 		} else if (!strcmp(argv[i], "--unstrap")) {
 			maxLastChar = UNSTRAPLIMIT;
 			warning(_("[unstrapped] you must start X11 with "
-				"\"-deferglyphs 16\" option!\n"));
+				"\"-deferglyphs 16\" option!"));
 		} else if (!strcmp(argv[i], "--daemon")) {
 			daemon = 1;
 		} else {
@@ -2102,7 +2152,7 @@ main(int argc, char **argv)
 
 	if (sync_db) {
 		if (ttSyncAll(gslist) <= 0)
-			error(_("sync failed.\n"));
+			error(_("sync failed"));
 		cleanupMem();
 		return 0;
 	}
@@ -2132,49 +2182,60 @@ main(int argc, char **argv)
 			pid_t pid = getpid();
 			fprintf(pidfile, "%d\n", pid);
 			fclose(pidfile);
-
-			signal_setup(SIGINT, delPIDfile);
-			signal_setup(SIGTERM, delPIDfile);
 		}
 	}
+	signal_setup(SIGINT, sigterm_handler);
+	signal_setup(SIGTERM, sigterm_handler);
 
-	int retry;
-	for (retry = 1; retry > 0; --retry) {
-		if (openTTFdb() > 0)
-			break;
+	if (openTTFdb() <= 0) {
 		closeTTFdb();
 
-		error(_("opening TTF database failed, while reading \"%s\" "
-		      "to build it.\n"), fontdir);
+		warning(_("opening font database failed, while reading \"%s\" "
+		          "to build it; trying to regenerate it"), fontdir);
 
-		if (ttSyncAll() > 0 && openTTFdb() > 0)
-			break;
-		error(_("creating a font database failed.\n"));
-		unlink(pidfilename);
+		if (ttSyncAll() <= 0) {
+			error(_("cannot regenerate font database"));
+			if (daemon)
+				unlink(pidfilename);
+			return 1;
+		}
+
+		if (openTTFdb() <= 0) {
+			error(_("cannot reopen regenerated font database"));
+			if (daemon)
+				unlink(pidfilename);
+			return 1;
+		}
 	}
 
 	signal_setup(SIGCHLD, SIG_IGN); // We don't need no stinkinig zombies -sjc
 	signal_setup(SIGHUP, sighup_handler);
 
-	fs_connection_setup(fs_conn);
+	if (fs_connection_setup(fs_conn) < 0)
+		return 1;
 
-	if (retry <= 0)
-		error(_("good bye.\n"));
-	else do {
+	debug("xfstt: server ready\n");
+
+	do {
 		fs_client client;
 
 		client.sd = inetdConnection ? 0 : fs_connection_new(fs_conn);
 
 		if (fs_connecting(client)) {
 			if (!multiConnection || !fork()) {
-				setuid(newuid);
-				setgid(newgid);
+				if (setuid(newuid) < 0) {
+					error(_("cannot set new user id"));
+					break;
+				}
+				if (setgid(newgid) < 0) {
+					error(_("cannot set new group id"));
+					break;
+				}
 
-				Rasterizer *raster = new Rasterizer();
+				Rasterizer raster;
 				client.replybuf = (char *)allocMem(MAXREPLYSIZE);
-				fs_working(client, raster);
+				fs_working(client, &raster);
 				deallocMem(client.replybuf, MAXREPLYSIZE);
-				delete raster;
 
 				if (!inetdConnection)
 					shutdown(client.sd, 2);
@@ -2186,8 +2247,7 @@ main(int argc, char **argv)
 
 				return 0;
 			} else if (multiConnection) {
-				// Redundant on most systems
-				// Needed for BSD Thanks David Lowe
+				// Redundant on most systems, needed for BSD.
 				int status;
 
 				waitpid(-1, &status, WNOHANG);
@@ -2196,12 +2256,8 @@ main(int argc, char **argv)
 		close(client.sd);
 	} while (multiConnection);
 
-	if (sockname) {
-		unlink(sockname);
-		rmdir(sockdir);
-	}
-
 	debug("xfstt: server down\n");
+	server_cleanup();
 	cleanupMem();
 
 	return 0;
